@@ -1,11 +1,13 @@
-#! perl
+#! perl --			-*- coding: utf-8 -*-
 
-# RCS Id          : $Id: BTWAangifte.pm,v 1.48 2008/07/21 14:56:06 jv Exp $
+use utf8;
+
+# RCS Id          : $Id: BTWAangifte.pm,v 1.50 2009/10/24 21:25:38 jv Exp $
 # Author          : Johan Vromans
 # Created On      : Tue Jul 19 19:01:33 2005
 # Last Modified By: Johan Vromans
-# Last Modified On: Mon Jul 21 16:07:15 2008
-# Update Count    : 637
+# Last Modified On: Sat Oct 24 14:43:49 2009
+# Update Count    : 642
 # Status          : Unknown, Use with caution!
 
 ################ Common stuff ################
@@ -20,7 +22,7 @@ package EB::Report::BTWAangifte;
 use strict;
 use warnings;
 
-our $VERSION = sprintf "%d.%03d", q$Revision: 1.48 $ =~ /(\d+)/g;
+our $VERSION = sprintf "%d.%03d", q$Revision: 1.50 $ =~ /(\d+)/g;
 
 use EB;
 use EB::Format;
@@ -227,7 +229,7 @@ sub periode {
     }
     elsif ( $p == 12 ) {
 	return __x("{month} {year}",
-		   month => $EB::month_names[$v-1],
+		   month => $EB::Utils::month_names[$v-1],
 		   year => $year);
     }
     else {
@@ -277,8 +279,8 @@ sub parse_periode {
     if ( $v =~ /^(\w+)$/i ) {
 	my $i;
 	for ( $i = 0; $i < 12; $i++ ) {
-	    last if lc($EB::month_names[$i]) eq lc($v);
-	    last if lc($EB::months[$i]) eq lc($v);
+	    last if lc($EB::Utils::month_names[$i]) eq lc($v);
+	    last if lc($EB::Utils::months[$i]) eq lc($v);
 	}
 	if ( $i < 12 ) {
 	    $pp->(12, $i+1);
@@ -403,7 +405,7 @@ sub collect {
 		    $deb_0 += $amt;
 		}
 		elsif ( $btg_id == BTWTARIEF_PRIV ) {
-		    $tr->("Priv");
+		    $tr->("Privé");
 		    $deb_p += $amt;
 		    $deb_btw_p += $btw;
 		}
@@ -774,7 +776,7 @@ sub kleine_ondernemers {
 
     # Formule lijkt linds 1995 ongwijzigd, alleen de bedragen
     # veranderen. Sinds 2002 zijn de bedragen ongewijzigd. Alle
-    # officile documentatie spreeekt over "Als de afdracht van
+    # officiële documentatie spreeekt over "Als de afdracht van
     # omzetbelasting beneden de 1.345 per jaar blijft, ...".
 
     # Code change: beschouw de huidige bedragen als vaststaand.
